/**
 * 文档: https://www.quanzhan.co/luch-request/
 * github: https://github.com/lei-mu/luch-request
 * DCloud: http://ext.dcloud.net.cn/plugin?id=392
 */

import Request from 'luch-request'
import Vue from '@/main'
import { getAuthorization, DEVICE_ID } from '@/utils/crypto'

// const getTokenStorage = () => {
//   let token = ''
//   try {
//     token = uni.getStorageSync('token')
//   } catch (e) {
//     console.log(e)
//   }
//   return token
// }

//先模拟PC端的登录
function setAuth(config) {
  let userSession = uni.getStorageSync('user')
  if (userSession && !config.custom?.noAuth) {
    let user = userSession
    const timestamp = Date.now()
    const authorization = getAuthorization(
      {
        method: config.method,
        uri: config.url.split('?')[0],
        timestamp,
        sessionId: user.sessionId,
        token: user.accessToken
      },
      'token'
    )
    config.header['Authorization'] = authorization
    config.header['time'] = timestamp
  }
}

const http = new Request()
http.setConfig((config) => {
  /* 设置全局配置 */
  config.baseURL = process.env.VUE_APP_BASE_API
  config.header = {
    ...config.header,
    platform: 'WEB',
    deviceId: DEVICE_ID
  }
  return config
})

http.interceptors.request.use(
  (config) => {
    setAuth(config)
    /* 请求之前拦截器。可以使用async await 做异步操作 */
    // config.header = {
    //   ...config.header,
    //   // token: getTokenStorage()
    //   Authorization: `Bearer ${Vue.vuex_token}`
    // }
    /*
       if (!token) { // 如果token不存在，return Promise.reject(config) 会取消本次请求
         return Promise.reject(config)
       }
       */
    /**
     * custom {Object} - 自定义参数
     */
    // if (config.custom.auth) {
    //   config.header.token = '123456'
    // }
    if (config.custom.loading) {
      uni.showLoading()
    }
    return config
  },
  (config) => {
    return Promise.reject(config)
  }
)

http.interceptors.response.use(
  async (response) => {
    if (response.config.custom.loading) {
      uni.hideLoading()
    }
    // if (response.data.code !== 200) { // 服务端返回的状态码不等于200，则reject()
    //   return Promise.reject(response)
    // }
    if (response.statusCode === 200) {
      return response.data?.data
    }
    if (response.statusCode === 401) {
      Vue.$u.vuex('vuex_token', '')
      Vue.$u.vuex('vuex_user_info', {})
      Vue.$u.toast('登陆过期')
      Vue.$u.route({
        url: 'pages/login/login'
      })
    } else if (response.statusCode === 500) {
      if (response.data.message) {
        Vue.$u.toast(response.data.message)
      } else {
        Vue.$u.toast('请求异常')
      }
    }
    return response
  },
  (err) => {
    console.log(err)
    if (response.config.custom.loading) {
      uni.hideLoading()
    }
    return Promise.reject(err)
  }
)

export { http }
